/*
 * test3_MPU6050
 * Robot prints out acceleration and gryo readings from MPU6050
 * as well as angles using the Elegoo Kalman Filter
 * 
 * Need to have "I2C dev" folder in Arduino libraries
 * Need to have "MPU6050" folder in Arduino libraries
 *
 */
/*********************************************************************/

#include "I2Cdev.h"
#include "MPU6050.h"
#include "KalmanFilter.h"

/*********************************************************************/

MPU6050 accelgyro;
KalmanFilter kalmanfilter;

// Current accelerometer and gyro zero values.
int16_t ax, ay, az;
int16_t gx, gy, gz;

int ax0 = 103;
int ay0 = 532;
// adjustment of az is not useful.
int gx0 = 355;
int gy0 = 196;
int gz0 = -220;

#define N_AVERAGES 7
double sums[N_AVERAGES];
unsigned long count = 0;

// Elegoo Kalman Filter parameters
float dt = 0.005;
float Q_angle = 0.001, Q_gyro = 0.005, R_angle = 0.5, C_0 = 1, K1 = 0.05;

unsigned int loop_count = 0;

/*********************************************************************/

void setup()
{
  Wire.begin();
  Serial.begin(115200);
  while( !Serial );
  Serial.println( "test3_MPU6050 version 30" );
  Serial.println( "Printing out accelerometer (a) and gyro (g) readings and averages" );
  Serial.println( "as well as angles computed from the Elegoo Kalman Filter." );
  Serial.println( "If I hang here, the MPU is in a bad state. Power cycle the robot." );
  delay(1000); // take time to allow above messages to print.
  accelgyro.initialize();
  Serial.println( "Initialized" );
  for ( int i = 0; i < N_AVERAGES; i++ )
    sums[i] = 0;
  delay(1000);
}

/*********************************************************************/

void loop()
{
  accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
  // subtract zeros
  ax -= ax0;
  ay -= ay0;
  // adjustment of az is not useful.
  gx -= gx0;
  gy -= gy0;
  gz -= gz0;
  // sum
  sums[0] += ax;
  sums[1] += ay;
  sums[2] += az;
  sums[3] += gx;
  sums[4] += gy;
  sums[5] += gz;
  ++count;

  kalmanfilter.Angle( ax, ay, az, gx, gy, gz,
		     dt, Q_angle, Q_gyro, R_angle, C_0, K1 );
  sums[6] += kalmanfilter.angle;

  delay(5);

  if ( ++loop_count < 100 )
    return;

  loop_count = 0;

  Serial.print(count);
  Serial.print(" a/g:\t");
  Serial.print(ax); Serial.print("\t");
  Serial.print(ay); Serial.print("\t");
  Serial.print(az); Serial.print("\t\t");
  Serial.print(gx); Serial.print("\t");
  Serial.print(gy); Serial.print("\t");
  Serial.print(gz); Serial.print("\tangle: ");
  Serial.println( kalmanfilter.angle );

  Serial.print(count);
  Serial.print(" ave:\t");
  Serial.print(sums[0]/count); Serial.print("\t");
  Serial.print(sums[1]/count); Serial.print("\t");
  Serial.print(sums[2]/count); Serial.print("\t");
  Serial.print(sums[3]/count); Serial.print("\t");
  Serial.print(sums[4]/count); Serial.print("\t");
  Serial.print(sums[5]/count); Serial.print("\tangle: ");
  Serial.println( sums[6]/count );

  Serial.println();
}

/*********************************************************************/
