// Test whether pins are seeing input from encoder lines
// Turn wheels and get numbers

#define ENCODER_LEFT_A_PIN 2
#define ENCODER_LEFT_B_PIN A7
#define ENCODER_RIGHT_A_PIN 4
#define ENCODER_RIGHT_B_PIN A6

volatile unsigned long int left_encoder_a_count = 0;
volatile unsigned long int left_encoder_b_count = 0;
volatile unsigned long int right_encoder_a_count = 0;
volatile unsigned long int right_encoder_b_count = 0;

void left_encoder_a_isr( void )
{
  left_encoder_a_count++;
}

void left_encoder_b_isr( void )
{
  left_encoder_b_count++;
}

void right_encoder_a_isr( void )
{
  right_encoder_a_count++;
}

void right_encoder_b_isr( void )
{
  right_encoder_b_count++;
}

void setup()
{
  pinMode( ENCODER_LEFT_A_PIN, INPUT_PULLUP );
  pinMode( ENCODER_LEFT_B_PIN, INPUT_PULLUP );
  pinMode( ENCODER_RIGHT_A_PIN, INPUT_PULLUP );
  pinMode( ENCODER_RIGHT_B_PIN, INPUT_PULLUP );

  Serial.begin(115200);
  delay( 100 );
  Serial.println("Nano Every interrupt test: Print each encoder line (A & B)");
  delay( 1000 ); // Provide time for loader to load new code

  attachInterrupt( digitalPinToInterrupt( ENCODER_LEFT_A_PIN ),
                                          left_encoder_a_isr, CHANGE);
  attachInterrupt( digitalPinToInterrupt( ENCODER_LEFT_B_PIN ),
                                          left_encoder_b_isr, CHANGE);
  attachInterrupt( digitalPinToInterrupt( ENCODER_RIGHT_A_PIN ),
                                          right_encoder_a_isr, CHANGE);
  attachInterrupt( digitalPinToInterrupt( ENCODER_RIGHT_B_PIN ),
                                          right_encoder_b_isr, CHANGE);
}


void loop()
{
  static int loop_counter = 0;

  delay(5);
  
  loop_counter++;

  if ( loop_counter%100 != 0 )
    return;

  Serial.print( "encoders: " );
  Serial.print( left_encoder_a_count );
  Serial.print( " " );
  Serial.print( left_encoder_b_count );
  Serial.print( "; " );
  Serial.print( right_encoder_a_count );
  Serial.print( " " );
  Serial.println( right_encoder_b_count );
}
