/******************************************************************/

#include "my_encoders.h"
#include "Motor.h"

/******************************************************************/

#define TARGET 500

/******************************************************************/

Motor Motor;

// how long have we been running? Safety feature
unsigned long loop_counter = 0;
unsigned long last_loop_counter = 0; 

static bool go = false;  /* motor enable */

/******************************************************************/

void setup()
{
  Serial.begin( 115200 );
  while( !Serial );
  Serial.println( "Servo test version 10" );
  delay( 1000 ); // Delay to make sure above message prints out.
  Motor.Motor_init();
  Serial.println( "Motor.Pin_init done." );
  Serial.println( "If we hang here power cycle the robot. Expect more printout" );
  delay( 1000 ); // Delay to make sure above messages print out.
  Encoder_init( &(Motor.left_angle), &(Motor.right_angle) );
  Serial.println( "Encoder_init done." );
  Serial.println( "Wheels should be off the ground."  );
  Serial.println( "Type g <return> to run test, s <return> to stop."  );
  Serial.println( "Typing window is at the top of the serial monitor window" );
}

/******************************************************************/

// Take user input
void ProcessCommand()
{

  if ( Serial.available() <= 0 )
    return;

  int c = Serial.read();
  switch (c)
    {
      case 'S': case 's':
        Serial.println( "Stop!" );
        go = false;
        break;
      case 'G': case 'g':
      Serial.println( "Go!" );
        go = true;
	      last_loop_counter = loop_counter;
        break;
      default:
        break;
    }
}

/******************************************************************/

// no timing on this loop. It just runs as fast as it can
void loop()
{
  static float angle_desired = 0.0;
  float k_pos = 1.0;

  loop_counter++;

  Read_encoders( &(Motor.left_angle), &(Motor.right_angle) );

  float left_error = Motor.left_angle - angle_desired;
  float right_error = Motor.right_angle - angle_desired;

  int left_command = -k_pos*left_error;
  if ( left_command > MAX_COMMAND )
    left_command = MAX_COMMAND;
  if ( left_command < -MAX_COMMAND )
    left_command = -MAX_COMMAND;

  int right_command = -k_pos*right_error;
  if ( right_command > MAX_COMMAND )
    right_command = MAX_COMMAND;
  if ( right_command < -MAX_COMMAND )
    right_command = -MAX_COMMAND;

  if ( loop_counter%1001 == 0 )
    {
       Serial.print( loop_counter );
       Serial.print( " " );
       Serial.print( go );
       Serial.print( " " );
       Serial.print( Motor.left_angle );
       Serial.print( " " );
       Serial.print( Motor.right_angle );
       Serial.print( "; " );
       Serial.print( left_error );
       Serial.print( " " );
       Serial.print( right_error );
       Serial.print( "; " );
       Serial.print( left_command );
       Serial.print( " " );
       Serial.println( right_command );
       return;
    }
  
  if ( loop_counter%1000 == 0 )
    ProcessCommand();

  if ( !go )
    {
      Motor.Stop();    
      return;
    }

  if ( loop_counter%21000 == 0 )
    {
      if ( angle_desired > 0 )
        angle_desired = 0;
      else
        angle_desired = TARGET;
    }
 
  // Turn off after a while to keep from running forever
  if ( (loop_counter - last_loop_counter) > 500000 )
    {
      Motor.Stop();
      Serial.println( "Motor stopped." );
      Serial.println( "Wheels should be off the ground."  );
      Serial.println( "Type g <return> to run test, s <return> to stop."  );
      go = false;
      return;
    }
    
  Motor.Left_command( left_command );
  Motor.Right_command( right_command );
}

/******************************************************************/
